/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.cli;

// for logging, and API access
import com.rempl.api.Logger;
import com.rempl.api.Model;
import com.rempl.api.ROM;
import com.rempl.api.Rempl;
import com.rempl.api.RemplFactory;
import com.rempl.api.Rempler;
import com.rempl.api.RuntimeProblem;

// for file listing
import java.io.File;
import java.io.StringWriter;

// CLI options
import org.apache.commons.cli.CommandLine;

/**
 * CLI request.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Main.java 42 2010-12-17 16:16:55Z yegor256@yahoo.com $
 */
class Request {

    /**
     * "rom" option that specifies the location of
     * <tt>rom.xml</tt> file.
     */
    public static final String OPT_ROM = "rom";

    /**
     * "xmi" option that specifies the location of
     * XMI file to be generated.
     */
    public static final String OPT_XMI = "xmi";

    /**
     * "report" option that specifies the location of
     * directory where reports will be saved to.
     */
    public static final String OPT_REPORT = "report";

    /**
     * The command line with options from CLI.
     */
    private CommandLine cmd;

    /**
     * Public ctor.
     * @param cline Command line
     * @see Main#main(String[])
     */
    public Request(final CommandLine cline) {
        this.cmd = cline;
    }

    /**
     * Execute request according to its configuration.
     * @throws Exception If something goes wrong
     * @todo #9 We should throw checked exception only in case
     *          of a problem with parameter. Then, the Main class
     *          has to catch it and respond with Helper. Otherwise
     *          we have to throw un-checked exception. I think that
     *          the checked exception has to be defined as a new
     *          class in the package.
     */
    public void run() throws Exception {
        Logger.info(this, "Report building started.");
        try {
            final RemplFactory factory = Rempl.getFactory();
            factory.setLoader(new HomeLoader());
            final ROM rom = new ROM(
                new File(this.cmd.getOptionValue(this.OPT_ROM))
            );
            final Rempler rempler = factory.makeRempler(rom);
            final Model model = rempler.read();
            final StringWriter writer = new StringWriter();
            model.save(writer);
            Logger.info(this, "Created %d bytes", writer.toString().length());
        } catch (java.io.IOException ex) {
            throw RuntimeProblem.make(ex);
        }
        Logger.info(this, "Report building finished.");
    }

}
